#!/usr/bin/python3

#
# SPDX-License-Identifier: Apache-2.0
#
# Copyright (C) 2022 Xilinx, Inc
#

import sys
import math

# ----------------------------------------------------------------------------
# VT100 ASCII Constants

ESC = '\x1b'          # Escape character
CSI = ESC + '['       # Control Sequence Introducer

CSHOW = CSI + "?25h"        # Show Cursor
CHIDE = CSI + "?25l"        # Hide Cursor
ALTSCREEN = CSI + "?1049h"  # Enable alternative screen buffer
NORMSCREEN = CSI + "?1049l" # Disable alternative screen buffer
ERASELINE = CSI + "2K"      # Erase Line

class ForegroundColor:
    reset = CSI + "0m"
    black = CSI + "30m"
    red = CSI + "31m"
    green = CSI + "32m"
    yellow = CSI + "33m"
    blue = CSI + "34m"
    magenta = CSI + "35m"
    cyan = CSI + "36m"
    white = CSI + "37m"
    lightblack = CSI + "90m"
    lightred = CSI + "91m"
    lightgreen = CSI + "92m"
    lightyellow = CSI + "93m"
    lightblue = CSI + "94m"
    lightmagenta = CSI + "95m"
    lightcyan = CSI + "96m"
    lightwhite = CSI + "97m"

fg = ForegroundColor()

class BackgroundColor:
    reset = CSI + "0m"
    black = CSI + "40m"
    red = CSI + "41m"
    green = CSI + "42m"
    yellow = CSI + "43m"
    blue = CSI + "44m"
    magenta = CSI + "45m"
    cyan = CSI + "46m"
    white = CSI + "47m"
    lightblack = CSI + "100m"
    lightred = CSI + "101m"
    lightgreen = CSI + "102m"
    lightyellow = CSI + "103m"
    lightblue = CSI + "104m"
    lightmagenta = CSI + "105m"
    lightcyan = CSI + "106m"
    lightwhite = CSI + "107m"

bg = BackgroundColor()

class Effects:
    reset = CSI + "0m"
    bold = CSI + "1m"
    dim = CSI + "2m"
    italic = CSI + "3m"
    underline = CSI + "4m"
    slowblink = CSI + "5m"
    rapidblink = CSI + "6m"
    reverse = CSI + "7m"

fx = Effects()

# Helper functions

def indented_print(term, lock, buf_list, loc_x, loc_y):
    with lock:
        for i, val in enumerate(buf_list):
            term.location(loc_x, loc_y + i)
            print(buf_list[i], flush=True, end='')

def print_section_heading(term, lock, heading, loc_y):
    with lock:
        term.location(0, loc_y)
        print(fg.blue + fx.bold + fx.italic + heading + fx.reset)

def print_warning(term, lock, loc_y, msg):
    with lock:
        term.location(5, loc_y)
        print(fg.yellow + msg + fg.reset)

def clear_rows(term, lock, start_row, num_lines):
    with lock:
        for row in range(start_row, start_row + num_lines):
            term.location(0, row)
            print(ERASELINE,  flush=True, end='')


def progress_bar(percent, width=24):
    import math
    segments = [b"\xe2\x96\xae".decode("utf-8"),
                b"\xe2\x96\xaf".decode("utf-8")]
    completed = math.floor(width * (percent / 100))

    bar = fg.green
    for index in range(0, width):
        if index < completed:
            bar += segments[0]
            continue

        if index == completed:
            bar += fg.blue

        bar += segments[1]

    bar += fg.white + (" %3.2f%%" % percent)
    return bar

def pad_string(padded_string, padding_width, alignment):
    # Alignment values: 'left', 'right', 'center'
    while True:
        if alignment != "right":
            if len(padded_string) < padding_width:
                padded_string = padded_string + " "
        if alignment != "left":
            if len(padded_string) < padding_width :
                padded_string = " " + padded_string
        if len(padded_string) >= padding_width:
            break

    return padded_string

def get_percentage(usage, size):
    if usage == 0:
        return "0 %"

    percent = usage * 100 / size

    return "%.2f %%" % percent

def convert_size(size_bytes):
    if size_bytes == 0:
        return "0  B"

    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)

    if i == 0:
        return ("%d  %s" % (s, size_name[i]))

    return "%.2f %s" % (s, size_name[i])


class Table:
    def __init__(self, header, data, format):
        self._header = header
        self._data = data
        self._format = format

    def _create_divider(self):
        line = "+"
        for col in range(len(self._header)):
            line = line + '-' * (len(self._header[col]) + 2) + "+"

        return line

    def _pad_header_and_data(self):
        # Examine each column looking for the maximum string length
        for col in range(len(self._header)):
            # Look for the maximum length
            max_length = len(self._header[col])
            for row in range(len(self._data)):
                if max_length < len(self._data[row][col]):
                    max_length = len(self._data[row][col])
            # Pad each entry
            self._header[col] = pad_string(self._header[col], max_length, "center")
            for row in range(len(self._data)):
                self._data[row][col] = pad_string(self._data[row][col], max_length, self._format[col])

    def _create_table(self):
        # Create the table
        table = []

        # Divider
        table.append(self._create_divider())

        # Add the header
        line = "|"
        for col in range(len(self._header)):
            line = line + " " + self._header[col] + " |"
        table.append(line)

        # Divider
        table.append(self._create_divider())

        # Add the data
        for row in range(len(self._data)):
            line = "|"
            for col in range(len(self._data[row])):
                line= line + " " + self._data[row][col] + " |"
            table.append(line)

        # Divider
        table.append(self._create_divider())

        return table

    def create_table(self):
        self._pad_header_and_data()
        return self._create_table()


class Terminal:
    # init method or constructor
    def __init__(self):
        self._stream = sys.stdout

    def __enter__(self):
        self._write_stream(ALTSCREEN)
        return self

    def __exit__(self, type, value, traceback):
        self.hide_cursor(False)
        self._write_stream(NORMSCREEN)

    def _write_stream(self, data):
        stream = self._stream
        stream.write(data)
        stream.flush()

    def hide_cursor(self, hide):
        if hide == True:
            self._write_stream(CHIDE)
        else:
            self._write_stream(CSHOW)

    def location(self, x_loc, y_loc):
        self._write_stream(CSI + "%d;%dH" % (y_loc, x_loc))


# ActiveState Code Recipe
# https://code.activestate.com/recipes/134892/
class _Getch:
    """Gets a single character from standard input.
       Does not echo to the screen.
    """
    def __init__(self):
        try:
            self.impl = _GetchWindows()
        except ImportError:
            self.impl = _GetchUnix()

    def __call__(self): return self.impl()


class _GetchUnix:
    def __init__(self):
        import tty

    def __call__(self):
        import sys, tty, termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setcbreak(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch


class _GetchWindows:
    def __init__(self):
        import msvcrt

    def __call__(self):
        import msvcrt
        return msvcrt.getch()

get_char = _Getch()



#!/usr/bin/python3

#
# SPDX-License-Identifier: Apache-2.0
#
# Copyright (C) 2022 Xilinx, Inc
#

import json
import math
from . import XBUtil

# found in PYTHONPATH
import pyxrt


class ReportMemory:
    # Python value used as second parameter for int() to convert hexadecimal strings into ints
    hexadecimal_converion = 16

    def report_name(self):
        return "Memory"

    def update(self, dev, report_length):
        self.report_length = report_length
        memory_json = dev.get_info(pyxrt.xrt_info_device.memory)
        memory_raw = json.loads(memory_json) #read into a dictionary

        # check if xclbin is loaded
        if not memory_raw:
            self._df = None

        self._df = memory_raw

        # Generate the number of pages used for the report
        try:
            # Round up the division to leave an extra page for the last batch of data
            page_count = math.ceil(len(self._df['board']['memory']['memories']) / report_length)
            # We must ensure that we always have at least one page
            self.topology_page_count = max(page_count, 1)
        except:
            self.topology_page_count = 0

        try:
            page_count = math.ceil(len(self._df['board']['direct_memory_accesses']['metrics']) / report_length)
            self.dma_page_count = max(page_count, 1)
        except:
            self.dma_page_count = 0

        self.page_count = max(self.topology_page_count + self.dma_page_count, 1)
        return self.page_count

    def _print_memory_usage(self, term, lock, start_x, start_y, page):
        XBUtil.print_section_heading(term, lock, "Device Memory Usage", start_y)
        offset = 1

        if self._df == None:
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        data = []
        memories = []
        try:
            memories = self._df['board']['memory']['memories']
        except:
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        for i in range(self.report_length):
            # The current element to be parsed depends on what page has been requested
            index = i + (page * self.report_length)
            # Ensure that our index does not exceed the input data size. This may happen on the last page
            if (index < len(memories)):
                memory = memories[index]
                size = int(memory['range_bytes'], self.hexadecimal_converion)
                if size == 0 or memory['enabled'] == "false":
                    continue
                name = memory['tag']
                usage = int(memory['extended_info']['usage']['allocated_bytes'], 0)
                data.append("%-16s %s" % (name, XBUtil.progress_bar(usage * 100 / size)))
            # If the index exceeds the input data size leave the for loop as everything is populated on the
            # last page
            else:
                break

        if (not data):
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable")
            return offset + 1

        XBUtil.indented_print(term, lock, data, 3, start_y + offset)
        offset += len(data)
        return offset

    def _print_mem_topology(self, term, lock, start_x, start_y, page):
        XBUtil.print_section_heading(term, lock, "Memory Topology", start_y)
        offset = 1

        if self._df == None:
            print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        header = [     "",  "Tag", "Type", "Temp (C)",  "Size", "Mem Usage", "Utilization", "BO Count"]
        format = ["right", "left", "left",    "right", "right",     "right",       "right",    "right"]
        data = []

        memories = []
        try:
            memories = self._df['board']['memory']['memories']
        except:
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        for i in range(self.report_length):
            line = []
            # The current data element to be parsed depends on what page has been requested
            index = i + (page * self.report_length)
            # Ensure that our index does not exceed the data size. This may happen on the last page
            if (index < len(memories)):
                line.append(str(index))
                memory = memories[index]
                line.append(memory['tag'])
                line.append(memory['type'])
                line.append(memory['extended_info']['temperature_C'] if 'temperature_C' in memory['extended_info'] else "--")
                line.append(XBUtil.convert_size(int(memory['range_bytes'], self.hexadecimal_converion)))
                line.append(XBUtil.convert_size(int(memory['extended_info']['usage']['allocated_bytes'],0)))
                
                size = int(memory['range_bytes'], self.hexadecimal_converion)
                if size == 0 or memory['enabled'] == "false":
                    line.append("--  ")
                else:
                    usage = int(memory['extended_info']['usage']['allocated_bytes'], 0)
                    line.append(XBUtil.get_percentage(usage, size))

                
                line.append(memory['extended_info']['usage']['buffer_objects_count'])
                data.append(line)
            # If the index exceeds the input data size leave the for loop as everything is populated on the
            # last page
            else:
                break

        if (not data):
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable")
            return offset + 1

        table = XBUtil.Table(header, data, format)
        ascii_table = table.create_table()

        XBUtil.indented_print(term, lock, ascii_table, 3, start_y + offset)
        offset += len(ascii_table)
        return offset

    def _print_dma_transfer_metrics(self, term, lock, start_x, start_y, page):
        XBUtil.print_section_heading(term, lock, "DMA Transfer Metrics", start_y)
        offset = 1

        if self._df == None:
            print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        header = [     "", "Channel", "Host-to-Card", "Card-to-Host"]
        format = ["right",   "right",        "right",        "right"]
        data = []

        dma_metrics = []
        try:
            dma_metrics = self._df['board']['direct_memory_accesses']['metrics']
        except:
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        for i in range(self.report_length):
            line = []
            # The current element to be parsed depends on what page has been requested
            index = i + (page * self.report_length)
            # Ensure that our index does not exceed the input data size. This may happen on the last page
            if (index < len(dma_metrics)):
                dma_metric = dma_metrics[index]
                line.append(str(i))
                line.append(dma_metric['channel_id'])
                line.append(XBUtil.convert_size(int(dma_metric['host_to_card_bytes'], self.hexadecimal_converion)))
                line.append(XBUtil.convert_size(int(dma_metric['card_to_host_bytes'], self.hexadecimal_converion)))
                data.append(line)
            # If the index exceeds the input data size leave the for loop as everything is populated on the
            # last page
            else:
                break

        if (not data):
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable")
            return offset + 1

        table = XBUtil.Table(header, data, format)
        ascii_table = table.create_table()

        XBUtil.indented_print(term, lock, ascii_table, 3, start_y + offset)
        offset += len(ascii_table)
        return offset

    def print_report(self, term, lock, start_x, start_y, page):
        offset = 0
        # The pages in order are:
        # 1. Memory Topology
        # 2. DMA tranfer data
        # The page values in the individual reports range from 0 - X and 
        # the input page refers to the total number of pages in the memory report 
        # Ex. (0 - (topology_page_count + dma_page_count)).
        # So the previous page count must be subtracted before passing the 
        # page parameter into the appropriate print function
        if (page < self.topology_page_count):
            offset = 1 + self._print_mem_topology(term, lock, start_x, start_y + offset, page)
        elif (page < self.page_count):
            offset = 1 + self._print_dma_transfer_metrics(term, lock, start_x, start_y + offset, page - self.topology_page_count)
        else:
            XBUtil.print_warning(term, lock, start_y + offset, "Something went wrong! Please report this issue and its conditions.")
        return offset




#!/usr/bin/python3

#
# SPDX-License-Identifier: Apache-2.0
#
# Copyright (C) 2022 Xilinx, Inc
#

import json
import math
from . import XBUtil

# found in PYTHONPATH
import pyxrt


class ReportDynamicRegions:
    def report_name(self):
        return "Dynamic Region"

    def update(self, dev, report_length):
        self.report_length = report_length
        #get cu info
        cu_json = dev.get_info(pyxrt.xrt_info_device.dynamic_regions)
        cu_raw = json.loads(cu_json) #read into a dictionary

        # Round up the division to leave an extra page for the last batch of data
        page_count_temp = math.ceil(len(cu_raw) / self.report_length)
        # We must ensure that we always have at least one page
        self.page_count = max(page_count_temp, 1)

        self._df = cu_raw
        return self.page_count

    def _print_cu_info(self, term, lock, start_x, start_y, page):
        XBUtil.print_section_heading(term, lock, "Compute Usage", start_y)
        table_offset = 1
        try:
            uuid = self._df['dynamic_regions'][0]['xclbin_uuid']
            cus = self._df['dynamic_regions'][0]['compute_units']
        except:
            XBUtil.print_warning(term, lock, start_y + table_offset, "Data unavailable. Acceleration image not loaded")
            return table_offset + 1

        with lock:
            term.location(3, start_y+table_offset)
            print("Xclbin UUID: %s" % uuid)
            table_offset += 2

        header = [     "", "Name", "Base Address", "Usage", "Status", "Type"]
        format = ["right", "left",        "right", "right", "center", "center"]
        data = []

        # Each page should display however many items a report page can hold
        for i in range(self.report_length):
            line = []
            # The current element to be parsed depends on what page has been requested
            index = i + (page * self.report_length)
            # Ensure that our index does not exceed the input data size. This may happen on the last page
            if(index < len(cus)):
                line.append(str(index))
                cus_element = cus[index]
                line.append(cus_element['name'])
                line.append(cus_element['base_address'])
                line.append(cus_element['usage'])
                line.append(cus_element['status']['bit_mask'])
                line.append(cus_element['type'])
                data.append(line)
            # If the index exceeds the input data size leave the for loop as everything is populated on the
            # last page
            else:
                break

        if len(data) != 0:
            table = XBUtil.Table(header, data, format)
            ascii_table = table.create_table()

            XBUtil.indented_print(term, lock, ascii_table, 3, start_y + table_offset)
            table_offset += len(ascii_table)

        return table_offset

    def print_report(self, term, lock, start_x, start_y, page):
        offset = 1 + self._print_cu_info(term, lock, start_x, start_y, page)
        return offset



#!/usr/bin/python3

#
# SPDX-License-Identifier: Apache-2.0
#
# Copyright (C) 2022 Xilinx, Inc
#

# Note: Only standard and local packages are permitted  in this script.

# Python Standard Packages
import argparse
import threading
import time
import json
from enum import IntEnum
from shutil import get_terminal_size

# XRT Package (Discovered in PYTHONPATH)
import pyxrt

# Local Packages
from _xbtop import XBUtil

# Global Variables
g_refresh_rate = 0
g_refresh_counter = 0
g_report_number = 0
g_page_number = 0

# Initialize the avaible reports
# Note: Once the report is added to the g_report array, it will automaticaly
#       be supported.
from _xbtop.ReportPower import ReportPower
from _xbtop.ReportMemory import ReportMemory
from _xbtop.ReportDynamicRegions import ReportDynamicRegions

g_reports = []
g_reports.append(ReportMemory())
g_reports.append(ReportDynamicRegions())
g_reports.append(ReportPower())


# Running clock thread (upper left corner)
def running_clock(term, lock):
    while True:
        with lock:
            term.location(0, 2)
            print(time.asctime(time.localtime(time.time())))

        time.sleep(0.5)

# Running counter for each of the reports (upper right corner)
def running_counter(term, lock, len_x):
    global g_refresh_rate
    global g_refresh_counter

    while True:
        with lock:
            # Add left 'padding' when counts and rates drop
            print_count = "     Refresh Count: %s (Rate: %s s)" % (g_refresh_counter, g_refresh_rate)
            term.location(len_x - len(print_count) - 2, 2)
            print(print_count)

        time.sleep(0.5)

# Key table
def print_footer(term, lock, y_len):
    footer_buf = [
        XBUtil.fg.yellow + "'p'/'n' - Previous/Next Report" + XBUtil.fg.reset,
        XBUtil.fg.yellow + "'<'/'>' - Dec/Inc Page" + XBUtil.fg.reset,
        XBUtil.fg.yellow + "    'q' - Quit" + XBUtil.fg.reset
    ]
    XBUtil.indented_print(term, lock, footer_buf, 5, y_len - len(footer_buf))

# Common platform information
def bdf_header(term, lock, dev):
    platform_json = dev.get_info(pyxrt.xrt_info_device.platform)
    platform_raw = json.loads(platform_json)
    platform = platform_raw['platforms'][0]['static_region']

    header_buf = [
        '  Shell : %s' % platform['vbnv'],
        '  UUID  : %s' % platform['logic_uuid'],
        '  BDF   : %s' % dev.get_info(pyxrt.xrt_info_device.bdf),
    ]
    longest_string = max(header_buf, key=len)
    header_buf.insert(0, '-' * (len(longest_string)))
    header_buf.append('-' * (len(longest_string)))
    XBUtil.indented_print(term, lock, header_buf, 0, 4)

# Prints both the BDF table and the current report being displayed
def print_header(term, lock, dev, report_header, page_header, len_x):
    # Add padding to overwrite previous report
    padding_width = 35
    report_header = XBUtil.pad_string(report_header, padding_width, "center")
    page_header = XBUtil.pad_string(page_header, padding_width, "center")

    with lock:
        center_align = int((len_x - len(report_header)) / 2)
        term.location(center_align, 2)
        print(XBUtil.fg.blue + XBUtil.fx.bold + XBUtil.fx.italic + report_header + XBUtil.fx.reset)

        center_align = int((len_x - len(page_header)) / 2)
        term.location(center_align, 3)
        print(XBUtil.fg.blue + XBUtil.fx.bold + XBUtil.fx.italic + page_header + XBUtil.fx.reset)

    bdf_header(term, lock, dev)

# Running thread used to driver the reports.
def running_reports(term, lock, dev, x_len, y_len):
    global g_report_number
    global g_reports
    global g_page_number

    report_length = 0
    report_start_row = 10
    report_end_row = 10 # Number of lines above bottom of terminal. Must leave room for footer!
    num_lines_printed = 0
    current_report = -1
    current_page = -1
    while True:
        global g_refresh_counter
        global g_refresh_rate

        g_refresh_counter += 1
        page_count = 0
        # Determine if our report has changed
        if (current_report != g_report_number) or (current_page != g_page_number):
            g_refresh_counter = 0

            # Clear the previous reports
            XBUtil.clear_rows(term, lock, report_start_row, num_lines_printed)

            # Point to the next report
            if g_report_number < 0:
                g_report_number = len(g_reports) - 1
            if g_report_number >= len(g_reports):
                g_report_number = 0
            current_report = g_report_number

            report_length = y_len - report_start_row - report_end_row
            page_count = g_reports[current_report].update(dev, report_length)

            # Point to the next page if overflowing
            if g_page_number < 0:
                g_page_number = page_count - 1
            if g_page_number >= page_count:
                g_page_number = 0
            current_page = g_page_number

            # Update the report header on which report that is currently being displayed
            report_name = g_reports[current_report].report_name()
            report_header = "%s (%d/%d)" % (report_name, current_report + 1, len(g_reports))
            page_header = "Page (%d/%d)" % (current_page + 1, page_count)
            print_header(term, lock, dev, report_header, page_header, x_len)
        # Just update the report if no changes have occurred
        else:
            page_count = g_reports[current_report].update(dev, report_length)

        # Clear the previous reports
        XBUtil.clear_rows(term, lock, report_start_row, num_lines_printed)
        num_lines_printed = g_reports[current_report].print_report(term, lock, 0, report_start_row, current_page)

        # Wait for either for the refresh time to expire or for a new report
        counter = 0
        while counter < (g_refresh_rate * 4):  # Note: The sleep time is 0.25 seconds, hence x4.
            counter += 1
            if current_report != g_report_number:
                break
            time.sleep(0.25)    # Check 4 times a second

# Parse the user's input
def options_parser():
    parser = argparse.ArgumentParser(description="xbtop: Process Viewer & Monitor")
    optional = parser._action_groups.pop()
    required = parser.add_argument_group('required arguments')
    required.add_argument("-d", "--device", dest = "bdf", help="The device bdf", required = True)
    optional.add_argument("-r", "--refresh_rate", dest = "s", default = 1, help="Refresh rate in seconds")
    parser._action_groups.append(optional)
    return parser.parse_args()

# Main processing thread
def main():
    global g_refresh_rate
    global g_report_number
    global g_page_number

    # Get and validate the options
    opt = options_parser()

    g_refresh_rate = float(opt.s)

    if g_refresh_rate < 1:
        raise RuntimeError("Please specify a refresh rate greater than 1 second")

    if g_refresh_rate > 60:
        raise RuntimeError("Please specify a refresh rate less than 60 seconds")

    dev = pyxrt.device(opt.bdf)

    # Check the terminal size
    term_size = get_terminal_size()
    x_len = term_size.columns
    y_len = term_size.lines
    MIN_X = 100
    MIN_Y = 44
    if x_len < MIN_X or y_len < MIN_Y:
        raise RuntimeError("Please resize the terminal window.  The current size %dx%d is smaller then the required size of %dx%d" % (x_len, y_len, MIN_X, MIN_Y))

    with XBUtil.Terminal() as term:
        term.hide_cursor(True)

        # creating a lock
        lock = threading.Lock()

        # Print the key
        print_footer(term, lock, y_len)

        # Running clock
        t1 = threading.Thread(target=running_clock, args=(term, lock))
        t1.daemon = True
        t1.start()

        # Running counter
        t1 = threading.Thread(target=running_counter, args=(term, lock, x_len))
        t1.daemon = True
        t1.start()

        # Running reports
        t2 = threading.Thread(target=running_reports, args=(term, lock, dev, x_len, y_len))
        t2.daemon = True
        t2.start()

        # Main thread that consumes the keys pressed by the user.
        while True:
            key = XBUtil.get_char()
            if key in ['q', 'Q']:
                break

            if key in ['<', ',']:
                g_page_number -= 1

            if key in ['>', '.']:
                g_page_number += 1

            if key in ['n', 'N']:
                g_report_number += 1
                g_page_number = 0

            if key in ['p', 'P']:
                g_report_number -= 1
                g_page_number = 0

            if key in ['+']:             # Hidden option
                g_refresh_rate += 1
                if g_refresh_rate > 60:
                    g_refresh_rate = 60

            if key in ['-']:             # Hidden option
                g_refresh_rate -= 1
                if g_refresh_rate < 1:
                    g_refresh_rate = 1


if __name__ == '__main__':
    try:
        main()

    except KeyboardInterrupt:
        pass

    except RuntimeError as r:
        print("ERROR: %s" % r)
        exit(1)

    except Exception as e:
        print("ERROR: %s" % e)
        exit(1)



#!/usr/bin/python3

#
# SPDX-License-Identifier: Apache-2.0
#
# Copyright (C) 2022 Xilinx, Inc
#

import json
import math
from . import XBUtil

# found in PYTHONPATH
import pyxrt


class ReportPower:

    def report_name(self):
        return "Power"

    def update(self, dev, report_length):
        self.report_length = report_length
        #get power info
        electrical_json = dev.get_info(pyxrt.xrt_info_device.electrical)
        electrical_raw = json.loads(electrical_json)  #read into a dictionary

        # check if xclbin is loaded
        if not electrical_raw:
            self._df = {}
            # If data is missing set the page count to 0!
            self.page_count = 0
            return self.page_count

        # dict with all power entries
        power_status = {}
        power_status['Max Power'] = electrical_raw['power_consumption_max_watts']
        power_status['Power'] = electrical_raw['power_consumption_watts']
        power_status['Warning'] = electrical_raw['power_consumption_warning']

        self._df = power_status
        # Round up the division to leave an extra page for the last batch of data
        page_count_temp = math.ceil(len(power_status) / self.report_length)
        # We must ensure that we always have at least one page
        self.page_count = max(page_count_temp, 1)
        return self.page_count

    def print_report(self, term, lock, start_x, start_y, page):
        XBUtil.print_section_heading(term, lock, self.report_name(), start_y)
        offset = 1

        if len(self._df) == 0 or self.page_count == 0:
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable. Acceleration image not loaded")
            return offset + 1

        # Create the complete power buffer
        all_data = [
            'Power     : %s Watts' % self._df['Power'],
            'Max Power : %s Watts' % self._df['Max Power'],
            'Warning   : %s' % self._df['Warning']
            ]

        # Extract the data elements to be displayed on the requested page
        page_offset = page * self.report_length
        # The upper offset is bounded by the size of the full power buffer
        upper_page_offset = min(self.report_length + page_offset, len(all_data))
        data = all_data[page_offset:upper_page_offset]

        if (not data):
            XBUtil.print_warning(term, lock, start_y + offset, "Data unavailable")
            return offset + 1

        XBUtil.indented_print(term, lock, data, 3, start_y + offset)
        offset += len(data)
        return offset
